using System;
using System.IO;
using System.Diagnostics;

namespace AVIInfo
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Class1
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
            if (args.Length < 1)
            {
                Usage(args);
                return;
            }

            Options options = new Options(args);

            if (options.bShowHelp)
            {
                Usage(args);
                return;
            }

            if (options.bVersion)
            {
                Console.WriteLine(Version);
                return;
            }

            if (options.Warnings)
                Console.Write(options.sWarnings);

            if (options.Errors)
                Console.Write(options.sErrors);

            if (!options.bValid)
                return;

            AVIWrapper avi = new AVIWrapper();

            avi.m_bDiagnostics = options.bDiagnostics;

            string sDirectory;
            string sPattern;

            if (options.bUseBSIFile)
            {
                sDirectory = Path.GetDirectoryName(options.sBSIFile);
                sPattern = Path.GetFileName(options.sBSIFile);
            }
            else
            {
                sDirectory = Path.GetDirectoryName(options.sFileName);
                sPattern = Path.GetFileName(options.sFileName);
            }


            if (sDirectory == "")
                sDirectory = ".";

            sDirectory = Global.EnsureSlash(sDirectory);

            string[] sFiles = Directory.GetFiles(sDirectory, sPattern);

            BSIWrapper bsiWrapper = new BSIWrapper();

            if (sFiles.Length < 1)
            {
                Console.WriteLine("No files were found that matched \"{0}\"", sPattern);
            }
            else
            {
                foreach(string sFile in sFiles)
                {
                    string sAVI = sFile;

                    if (options.bUseBSIFile)
                    {
                        if (!bsiWrapper.SetFile(sFile))
                        {
                            Console.WriteLine("The BSI file \"{0}\" could not be parsed properly.", sFile);
                            return;
                        }
                        sAVI = bsiWrapper.AVIFile;
                    }

                    if (!avi.SetAVI(sAVI))
                    {
                        Console.Write(avi.Warnings);
                        Console.Write(avi.Errors);

                        return;
                    }

                    if (options.bBasicInfo)
                    {
                        Console.WriteLine(avi.BasicInfo());
                    }

                    if (options.bAdvancedInfo)
                    {
                        Console.WriteLine(avi.AdvancedInfo());
                    }

                    if (options.bKeyFrames)
                    {
                        long[] KeyFrames = avi.Keyframes;

                        foreach(long frame in KeyFrames)
                        {
                            Console.WriteLine("{0}", frame);
                        }
                    }

                    if (options.bWarnEndKeyframe)
                    {
                        if (avi.LastKeyframe == avi.FrameCount - 1)
                        {
                            Console.WriteLine("{0}: Last frame ({1}) is a keyframe", sAVI, avi.LastKeyframe);
                        }
                    }

                    if (options.bUseBSIFile)
                    {
                        if (bsiWrapper.RoundChapterFrames(avi.Keyframes))
                            bsiWrapper.WriteNewFile();
                    }
                }
            }
		}

        static void Usage(string[] args)
        {
            Console.WriteLine(Version);
            Console.WriteLine();
            Console.WriteLine(Options.Usage);
        }

        static string Version
        {
            get
            {
                return "AVIInfo version " + Process.GetCurrentProcess().MainModule.FileVersionInfo.ProductVersion;
            }
        }
	}
}
